package com.trade.client.sales.service.impl;

import cn.hutool.core.collection.CollectionUtil;
import com.trade.client.consumer.domain.CustomerUser;
import com.trade.client.consumer.service.CustomerUserService;
import com.trade.client.sales.dto.SalesContractInfoDto;
import com.trade.client.sales.service.SalesService;
import com.trade.common.constant.HttpStatus;
import com.trade.common.enums.ContractStatus;
import com.trade.common.enums.SubmitStatus;
import com.trade.common.exception.ValidatorException;
import com.trade.common.utils.uuid.BizNoUtil;
import com.trade.common.validate.ValidatorGroup;
import com.trade.common.validate.ValidatorUtil;
import com.trade.service.FileTypeEnum;
import com.trade.service.contract.dto.ContractGenInputItem;
import com.trade.service.contract.dto.ContractGenOutputItem;
import com.trade.service.contract.service.ContractService;
import com.trade.service.filestorage.entity.TFileStorage;
import com.trade.service.filestorage.enums.FileModuleEnum;
import com.trade.service.filestorage.service.LocalFileService;
import com.trade.service.filestorage.util.MultipartFileUtil;
import com.trade.service.sales.domain.*;
import com.trade.service.sales.dto.ContractFileDto;
import com.trade.service.sales.dto.ContractTermsDto;
import com.trade.service.sales.dto.SalesCommodityDto;
import com.trade.service.sales.dto.SalesContractDto;
import com.trade.service.sales.service.*;
import com.trade.service.sales.service.impl.SalesDataConverter;
import com.trade.service.sales.vo.SalesContractInfoVo;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.multipart.MultipartHttpServletRequest;

import javax.annotation.Resource;
import javax.servlet.http.HttpServletRequest;
import java.io.IOException;
import java.time.LocalDateTime;
import java.util.ArrayList;
import java.util.List;

/**
 * @description: 销售服务实现类
 * @Author: wanglang
 * @Date: 2024/9/3 19:14
 * @Version: 1.0
 */
@Service
public class SalesServiceImpl implements SalesService {

    @Resource
    private CustomerUserService customerUserService;
    @Resource
    private SalesContractService salesContractService;
    @Resource
    private SalesDataConverter salesDataConverter;
    @Resource
    private ContractFileService contractFileService;
    @Resource
    private LocalFileService localFileService;
    @Resource
    private ContractService contractService;
    @Resource
    private SalesCommodityService salesCommodityService;
    @Resource
    private ContractTermsService contractTermsService;
    @Resource
    private ContractTermsExtendService contractTermsExtendService;

    @Override
    @Transactional(rollbackFor = Exception.class)
    public Long saveSalesContract(SalesContractInfoDto salesContractInfoDto, HttpServletRequest request) {
        CustomerUser customerUser = customerUserService.getCustomerUser();
        SalesContractDto salesContractDto = salesContractInfoDto.getSalesContract();
        MultipartFile transferFile = null;
        if (request instanceof MultipartHttpServletRequest) {
            transferFile = ((MultipartHttpServletRequest) request).getFile("file");
        }
        // 设置为草稿状态
        salesContractDto.setSubmitStatus(SubmitStatus.DRAFT.getCode());
        SalesContract contractNoExist = salesContractService.lambdaQuery()
                .eq(SalesContract::getContractNo, salesContractDto.getContractNo()).one();
        // 校验参数
        checkSalesContractParameter(salesContractDto, transferFile, contractNoExist);
        boolean isTemplate = false;
        ContractFile contractFile = handleContractFile(salesContractDto, customerUser, transferFile, isTemplate);
        salesContractDto.setContractNo(contractFile.getContractNo());
        salesContractDto.setContractStatus(ContractStatus.DRAFT.getCode());
        SalesContract salesContract = salesDataConverter.convertSalesContractDo(salesContractDto);
        // 更新时 这个字段不能为空
        if (salesContract.getId() != null) {
            salesContractService.updateById(salesContract);
            if (!salesContractDto.getTemplateType()) {
                ContractFile contractFileExist = contractFileService.lambdaQuery()
                        .eq(ContractFile::getContractNo, contractFile.getContractNo()).one();
                contractFile.setId(contractFileExist.getId());
                contractFileService.updateById(contractFile);
            }
        } else {
            // 1、保存合同信息
            salesContractService.save(salesContract);
            if (!salesContractDto.getTemplateType()) {
                // 2、保存销售合同文件信息
                contractFileService.save(contractFile);
            }
        }
        // 3、保存商品信息
        List<SalesCommodity> salesCommodities = salesCommodityService.lambdaQuery()
                .eq(SalesCommodity::getSalesContractId, contractFile.getId()).list();
        if (CollectionUtil.isNotEmpty(salesCommodities)) {
            salesCommodityService.removeBatchByIds(salesCommodities);
        }
        List<SalesCommodityDto> salesCommodityDtos = salesContractInfoDto.getSalesCommoditys();
        salesCommodityDtos.forEach(salesCommodityDto -> {
            salesCommodityDto.setSalesContractId(salesContract.getId());
        });
        salesCommodityService.saveBatch(salesDataConverter.convertCommodityDo(salesCommodityDtos));
        return salesContract.getId();
    }

    @Override
    public SalesContractInfoVo getSalesContractBaseInfo(Long contractId) {
        return salesContractService.getSalesContractDetails(contractId);
    }

    @Override
    @Transactional(rollbackFor = Exception.class)
    public boolean saveContractTerms(ContractTermsDto contractTermsDto) {
        if (SubmitStatus.FORMAL.getCode().equals(contractTermsDto.getSubmitStatus())) {
            ValidatorUtil.validator(ValidatorGroup.ValidatorType.SUBMIT, contractTermsDto);
        }
        ContractTerms contractTermsExist = contractTermsService.lambdaQuery()
                .eq(ContractTerms::getContractNo, contractTermsDto.getContractNo()).one();
        ContractTerms contractTerms = salesDataConverter.convertContractTermsDo(contractTermsDto);
        if (contractTermsExist != null) {
            contractTerms.setId(contractTermsExist.getId());
            contractTermsService.updateById(contractTerms);
        } else {
            contractTermsService.save(contractTerms);
        }
        List<ContractTermsExtend> contractTermsExtendsExist = contractTermsExtendService.lambdaQuery()
                .eq(ContractTermsExtend::getContractTermsId, contractTerms.getId()).list();
        if (CollectionUtil.isNotEmpty(contractTermsExtendsExist)) {
            contractTermsExtendService.removeBatchByIds(contractTermsExtendsExist);
        }
        contractTermsExtendService.saveBatch(contractTermsDto.getPaymentInfo());
        contractTermsExtendService.saveBatch(contractTermsDto.getTitleInfo());
        if (SubmitStatus.FORMAL.getCode().equals(contractTermsDto.getSubmitStatus())) {
            SalesContract salesContract = salesContractService.lambdaQuery()
                    .eq(SalesContract::getContractNo, contractTermsDto.getContractNo()).one();
            salesContract.setContractStatus(ContractStatus.TAKE_EFFECT.getCode());
            salesContract.setSubmitStatus(SubmitStatus.FORMAL.getCode());
            salesContractService.updateById(salesContract);
        }
        return true;
    }

    /**
     * 校验销售合同参数
     * 
     * @param salesContractDto
     * @param transferFile
     * @param salesContractExist
     */
    private void checkSalesContractParameter(SalesContractDto salesContractDto, MultipartFile transferFile,
            SalesContract salesContractExist) {
        if (!salesContractDto.getTemplateType()) {
            if (salesContractDto.getContractNo() == null || salesContractDto.getSignDate() == null) {
                throw new ValidatorException(HttpStatus.ERROR, "合同编号与签署日期不能为空！");
            }
            if (transferFile == null) {
                throw new ValidatorException(HttpStatus.ERROR, "请您上传合同！");
            }
            // 获取文件名
            String filename = transferFile.getOriginalFilename();
            String suffix;
            int dotIndex = filename.lastIndexOf('.');
            if (dotIndex == -1 || dotIndex == filename.length() - 1) {
                suffix = "";
            } else {
                suffix = filename.substring(dotIndex + 1);
            }
            if ("pdf".equalsIgnoreCase(suffix)) {
                throw new ValidatorException(HttpStatus.ERROR, "合同文件格式必须为pdf格式");
            }
            boolean meetLimit = MultipartFileUtil.checkFileSize(transferFile, 2, "M");
            if (!meetLimit) {
                throw new ValidatorException(HttpStatus.ERROR, "合同文件大小请控制在20M以内");
            }
        }
        if (salesContractExist != null && SubmitStatus.FORMAL.getCode().equals(salesContractExist.getSubmitStatus())) {
            throw new ValidatorException(HttpStatus.ERROR, "该合同编号已存在正式合同，请勿重复提交！");
        }
    }

    /***
     * 处理合同文件
     * 
     * @param salesContractDto
     * @param customerUser
     * @param transferFile
     * @param isTemplate
     * @return 合同文件信息
     */
    private ContractFile handleContractFile(SalesContractDto salesContractDto, CustomerUser customerUser,
            MultipartFile transferFile, boolean isTemplate) {
        try {
            ContractFile contractFile = new ContractFile();
            contractFile.setSignFileType(FileTypeEnum.SALES_CONTRACT.getCode());
            contractFile.setGenerationTime(LocalDateTime.now());
            contractFile.setOperatorName(customerUser.getNickname());
            if (salesContractDto.getTemplateType() && isTemplate) {
                // 生成合同文件
                contractFile.setContractNo(BizNoUtil.generateSalesContractNo());
                // ContractGenInput input = new ContractGenInput();
                // input.setBizNo(contractFile.getContractNo());
                // input.setUserId(customerUser.getId());
                // input.setBizType(FileTypeEnum.SALES_CONTRACT.getCode());
                ContractGenInputItem contractGenInputItem = new ContractGenInputItem();
                contractGenInputItem.setContractName(salesContractDto.getContractName());
                // todo 缺失合同模板
                ContractGenOutputItem contractGenOutput = contractService.generateSingleContract(contractGenInputItem,
                        customerUser.getId());
                contractFile.setUnsignFileStorageId(contractGenOutput.getFileId());
            } else {
                TFileStorage fileStorage = localFileService.addStorage(transferFile, FileModuleEnum.UNSIGNED_CONTRACT);
                contractFile.setContractNo(salesContractDto.getContractNo());
                contractFile.setUnsignFileStorageId(fileStorage.getId());
                contractFile.setSignedFileStorageId(fileStorage.getId());
            }
            return contractFile;
        } catch (IOException e) {
            throw new ValidatorException(HttpStatus.ERROR, "合同文件处理失败，请稍后再试！");
        }
    }
}
