/**
 * @filename:SalesCommodityDto 2024-09-03
 * @project KJT v1.0.0 Copyright(c) 2024 wanglang Co. Ltd. All right reserved.
 */
package com.trade.service.sales.dto;

import java.math.BigDecimal;

import javax.validation.Valid;
import javax.validation.constraints.NotBlank;

import com.trade.common.core.domain.CommonEntity;
import com.trade.common.validate.ValidatorGroup;

import io.swagger.annotations.ApiModelProperty;
import lombok.Data;
import lombok.EqualsAndHashCode;
import lombok.experimental.Accessors;

/**
 * @Description:销售商品实体类
 * @version: v1.0.0
 * @author: wanglang
 */
@Data
@EqualsAndHashCode(callSuper = false)
@Accessors(chain = true)
@Valid
public class SalesCommodityDto {

    private static final long serialVersionUID = 1725342360664L;
    /**
     * 销售合同主键
     */
    @ApiModelProperty(name = "salesContractId", value = "销售合同主键")
    @NotBlank(message = "销售合同主键不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    private Long salesContractId;
    /**
     * 商品编码
     */
    @ApiModelProperty(name = "hsCode", value = "商品编码")
    @NotBlank(message = "商品编码不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    private String hsCode;
    /**
     * 商品名称
     */
    @ApiModelProperty(name = "name", value = "商品名称")
    @NotBlank(message = "商品名称不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    private String name;
    /**
     * 包装规格
     */
    @ApiModelProperty(name = "packSpec", value = "包装规格")
    @NotBlank(message = "包装规格不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    private String packSpec;
    /**
     * 计量单位
     */
    @ApiModelProperty(name = "measurementUnit", value = "计量单位")
    @NotBlank(message = "计量单位不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    private String measurementUnit;
    /**
     * 出口退税率
     */
    @ApiModelProperty(name = "exportTaxRebateRate", value = "出口退税税率")
    @NotBlank(message = "出口退税税率不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    private String exportTaxRebateRate;
    /**
     * 数量
     */
    @NotBlank(message = "数量不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    @ApiModelProperty(name = "number", value = "数量")
    private Integer number;
    /***
     * 单价
     */
    @NotBlank(message = "单价不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    @ApiModelProperty(name = "priceUnit", value = "单价")
    private BigDecimal priceUnit;
    /**
     * 金额
     */
    @ApiModelProperty(name = "amount", value = "金额")
    @NotBlank(message = "金额不能为空", groups = {
            ValidatorGroup.Submit.class
    })
    private BigDecimal amount;
}
